package ru.landgrafhomyak.serdha.api.v0.misc

import kotlin.jvm.JvmName
import ru.landgrafhomyak.serdha.api.v0.table.RowId
import ru.landgrafhomyak.serdha.api.v0.table.Table

/**
 * Provider of database types descriptors.
 */
@Suppress("FunctionName", "PropertyName")
public interface DatabaseTypesProvider {
	/**
	 * Type of internal row id. Can't be casted to types like integers or pointers.
	 * @param TableUserExtension User's extension type for target table. For static type checking.
	 * @param table Descriptor of table whose row_id type will be obtained.
	 */
	@Suppress("INAPPLICABLE_JVM_NAME")
	@JvmName("ROW_ID")
	public fun <TableUserExtension : Any> ROW_ID(table: Table<TableUserExtension, *>): DatabaseType<RowId<TableUserExtension>>

	@Suppress("INAPPLICABLE_JVM_NAME")
	@JvmName("NULLABLE_ROW_ID")
	public fun <TableUserExtension : Any> NULLABLE_ROW_ID(table: Table<TableUserExtension, *>): DatabaseType<RowId<TableUserExtension>>

	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("BOOLEAN")
	public val BOOLEAN: DatabaseType.BOOLEAN

	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("NULLABLE_BOOLEAN")
	public val NULLABLE_BOOLEAN: DatabaseType.NULLABLE_BOOLEAN

	/**
	 * Descriptor of a raw binary data type without size restrictions (must be controlled by user).
	 */
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("BINARY_DATA")
	public val BINARY_DATA: DatabaseType.BINARY_DATA

	/**
	 * Descriptor of a raw binary data type that can holds maximum [size] bytes.
	 *
	 * @param size Maximum count of bytes that can be stored in the value of this type.
	 * @param isFixedSize If `true`, the type will restrict values with less than [size] bytes.
	 */
	@Suppress("INAPPLICABLE_JVM_NAME")
	@JvmName("BINARY_DATA")
	public fun BINARY_DATA(size: UInt, isFixedSize: Boolean): DatabaseType.BINARY_DATA
}