package ru.landgrafhomyak.serdha.api.v0.queries

import ru.landgrafhomyak.serdha.api.v0.misc.Column
import ru.landgrafhomyak.serdha.api.v0.misc.IntermediateColumn
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseType
import ru.landgrafhomyak.serdha.api.v0.misc.DatabaseTypesProvider
import ru.landgrafhomyak.serdha.api.v0.misc.Expression
import ru.landgrafhomyak.serdha.api.v0.misc.QueryParam
import ru.landgrafhomyak.serdha.api.v0.misc._Selectable
import ru.landgrafhomyak.serdha.api.v0.runtime.ParametersSetter

public interface SelectQuery<QueryUserExtension : Any> : _Selectable<QueryUserExtension> {
	public val userExtension: QueryUserExtension

	public enum class Order {
		ASC, DESC
	}

	@Suppress("ClassName")
	public interface _CommonSelectCreatorScope<QueryUserExtension : Any> {
		public val types: DatabaseTypesProvider
		public fun <RT, DT : DatabaseType<RT>> queryParam(name: String, type: DT): QueryParam<RT, DT, QueryUserExtension>
		public fun <RT, DT : DatabaseType<RT>> paramAsColumn(param: QueryParam<RT, DT, QueryUserExtension>): IntermediateColumn<RT, DT, QueryUserExtension>

		public fun <ExpressionUserExtension : Any> mapColumns(
			expression: Expression<ExpressionUserExtension>,
			input: Expression.InputLinker<ExpressionUserExtension, QueryUserExtension>,
			output: Expression.OutputLinker<ExpressionUserExtension, QueryUserExtension>
		)

		public fun where(column: IntermediateColumn<Boolean, DatabaseType.BOOLEAN, QueryUserExtension>)
		public fun orderBy(order: Order, vararg column: IntermediateColumn<*, *, QueryUserExtension>)

		public var limit: UInt
		public var offset: ULong

		public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, param: QueryParam<RT, DT, QueryUserExtension>): Column<RT, DT, QueryUserExtension>
		public fun <RT, DT : DatabaseType<RT>> returnColumn(name: String, column: IntermediateColumn<RT, DT, QueryUserExtension>): Column<RT, DT, QueryUserExtension>
	}

	public interface WithJoin<QueryUserExtension : Any> : SelectQuery<QueryUserExtension> {
		public interface JoinCreator<QueryUserExtension : Any, LeftTableUserExtension : Any, RightTableUserExtension : Any> : _CommonSelectCreatorScope<QueryUserExtension> {
			public fun createSelectWithJoin(creator: Scope<QueryUserExtension, LeftTableUserExtension, RightTableUserExtension>): QueryUserExtension

			public interface Scope<QueryUserExtension : Any, LeftTableUserExtension : Any, RightTableUserExtension : Any> {
				public val leftSubqueryParams: ParametersSetter.WithRedirect<LeftTableUserExtension, QueryUserExtension>
				public val rightSubqueryParams: ParametersSetter.WithRedirect<RightTableUserExtension, QueryUserExtension>

				public fun <RT, DT : DatabaseType<RT>> selectColumnFromLeft(column: Column<RT, DT, LeftTableUserExtension>): IntermediateColumn<RT, DT, QueryUserExtension>
				public fun <RT, DT : DatabaseType<RT>> selectColumnFromRight(column: Column<RT, DT, RightTableUserExtension>): IntermediateColumn<RT, DT, QueryUserExtension>
			}
		}
	}

	public interface FromSubquery<QueryUserExtension : Any> : InsertQuery<QueryUserExtension> {
		public interface Creator<QueryUserExtension : Any, TableUserExtension : Any> {
			public fun createSelect(creator: Scope<QueryUserExtension, TableUserExtension>): QueryUserExtension

			public interface Scope<QueryUserExtension : Any, TableUserExtension : Any> : _CommonSelectCreatorScope<QueryUserExtension> {
				public val subqueryParams: ParametersSetter.WithRedirect<TableUserExtension, QueryUserExtension>

				public fun <RT, DT : DatabaseType<RT>> selectColumnFromSubquery(param: Column<RT, DT, TableUserExtension>): IntermediateColumn<RT, DT, QueryUserExtension>
			}
		}
	}
}